import Component from '@ember/component'
import { htmlSafe } from '@ember/string'
import { set } from '@ember/object'
import { tryInvoke } from '@ember/utils'

import layout from '../templates/components/en-dropdown'
import DocumentEvent from 'en-select/mixins/document-event'
import KeyMap from 'en-select/utils/keymap'

export default Component.extend(DocumentEvent, {
  layout,

  classNames: ['dropdown-container'],
  classNameBindings: ['isFixed:is-fixed', 'isDisabled:isDisabled'],
  documentEvents: ['click'],

  isOpen: false,
  isFixed: false,
  optionsStyle: htmlSafe(''),

  onDocumentClick(e) {
    if (!this.isOpen) return
    let content = this.element.querySelector('.dropdown-content')

    if (
      !content ||
      !(content instanceof HTMLElement) ||
      content.contains(e.target)
    ) {
      return
    }

    this.closeDropdown()
  },

  keyUp(e) {
    let { isOpen } = this
    let { which, keyCode } = e
    let key = which || keyCode

    switch(key) {
      case KeyMap.esc:
        if (isOpen) this.closeDropdown()
        break;

      default:
        break;
    }
  },

  closeDropdown() {
    set(this, 'isOpen', false)
    tryInvoke(this, 'on-close')
  },

  openDropdown() {
    set(this, 'isOpen', true)
    tryInvoke(this, 'on-open')
  },
})
