import EmberObject, {
  setProperties,
  set,
  get
} from "@ember/object"
import stripKeyword from 'en-select/utils/strip-keyword'

let cleanup = word => {
  return stripKeyword(word).toLowerCase()
}

let pushToTrie = (trie = {}, word, id, idx) => {
  if (!word) return

  let letter = word[idx]
  if (!letter) return

  if (!trie[letter]) {
    trie[letter] = {
      ids: []
    }
  }

  trie[letter]["ids"].push(id)

  return pushToTrie(trie[letter], word, id, idx + 1)
}

let searchFor = (word, trie) => {
  const letters = word
    .replace(/ /g, "")
    .toLowerCase()
    .split("")

  let subTrie = trie
  let lastElement = false

  for(let i = 0; i < letters.length; i++) {
    let letter = letters[i]

    // If the subtrie does not have the letter, we break. This ensures that we
    // don't search for parts. Either the whole word exists as-is or it doesn't.
    if (!subTrie[letter]) {
      break;
    }

    // If we are at the last element, that means we've got the whole word. If we
    // don't check for this, the search results will be more lenient showing
    // items that have part(s) of the searched term.
    if (i === letters.length - 1) {
      lastElement = true
    }

    subTrie = subTrie[letter]
  }

  // If the lastElement exists, it means we have the whole word, so we return
  // the IDs from the subtree, otherwise we return an empty array.
  if (lastElement) {
    return subTrie.ids
  } else {
    return []
  }
}

export default EmberObject.extend({
  init() {
    this._super(...arguments)

    setProperties(this, {
      options: [],
      index: {
        ids: []
      }
    })
  },

  buildIndexFor(options, labelPath) {
    const index = get(this, "index")

    const mapToSearchable = (option) => ({
      label: get(option, labelPath),
      id: get(option, "id")
    })

    options
      .map(mapToSearchable)
      .filter(searchable => searchable.label)
      .forEach(({ label, id }) => {
        if (!label) return
        label.split(" ").forEach(word => {
          pushToTrie(index, cleanup(word), id, 0)
        })
      })

    set(this, "index", index)
    set(this, "options", options)
  },

  searchFor(term = "", minimumScore = 0.66) {
    const index = get(this, "index")
    const options = get(this, "options")

    let results = {}

    cleanup(term).split(" ").forEach(part => {
      let ids = searchFor(part, index)

      ids.forEach(id => {
        if (!results[id]) results[id] = 0
        results[id]++
      })
    })

    const ids = Object.keys(results)
    const highestScore = Math.max(...ids.map(id => results[id]))

    for (let id in results) {
      let relativeScore = results[id] / highestScore

      if (relativeScore < minimumScore) {
        delete results[id]
      }
    }

    return options.filter(option => {
      return Object.keys(results).includes(get(option, "id").toString())
    })
  }
})
